<?php

/**
 * The admin-specific functionality of the plugin.
 * @link       http://www.travelpayouts.com/?locale=en
 * @since      1.0.0
 * @package    Travelpayouts
 * @subpackage Travelpayouts/admin
 */

namespace Travelpayouts\admin;

use DI\Annotation\Inject;
use Exception;
use RuntimeException;
use Travelpayouts;
use Travelpayouts\admin\components\landingPage\LandingModel;
use Travelpayouts\admin\controllers\WidgetPreviewController;
use Travelpayouts\admin\partials\LandingPage;
use Travelpayouts\admin\redux\OptionsInit;
use Travelpayouts\components\BaseInjectedObject;
use Travelpayouts\components\Menu;
use Travelpayouts\components\Notices;
use Travelpayouts\includes\migrations\Migration;
use Travelpayouts\components\snowplow\TrackerHelper;
use Travelpayouts\components\LanguageHelper;

/**
 * The admin-specific functionality of the plugin.
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 * @package    Travelpayouts
 * @subpackage Travelpayouts/admin
 * @author     travelpayouts < wpplugin@travelpayouts.com>
 */
class Admin extends BaseInjectedObject
{
    /**
     * The ID of this plugin.
     * @since    1.0.0
     * @access   private
     * @var      string $plugin_name The ID of this plugin.
     * @Inject("name")
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     * @since    1.0.0
     * @access   private
     * @var      string $version The current version of this plugin.
     * @Inject("version")
     */
    private $version;

    public function setUpHooks()
    {
        $loader = Travelpayouts::loader();

        $loader->add_action('wp_ajax_travelpayouts_widget_render',
            WidgetPreviewController::getInstance(),
            'run'
        );
        /** @see Admin::localize_scripts() */
        $loader->add_action('admin_enqueue_scripts', $this, 'localize_scripts');
        /** @see Admin::importFile() */
        $loader->add_action('wp_ajax_travelpayouts_import_file', $this, 'importFile');
        /** @see Admin::migrate() */
        $loader->add_action('wp_ajax_travelpayouts_migrate', $this, 'migrate');
        /** @see Admin::migrateCancel() */
        $loader->add_action('wp_ajax_travelpayouts_migrate_cancel', $this, 'migrateCancel');
        /** @see Admin::load_redux_options() */
        $loader->add_action('plugins_loaded', $this, 'load_redux_options');
        /** @see Admin::add_page() */
        $loader->add_action('admin_menu', $this->get_landing_page(), 'add_page');
        /** @see Admin::admin_notices() */
        $loader->add_action('admin_notices', $this, 'admin_notices');
        /** @see Admin::overrideReduxCss() */
        $loader->add_action('redux/page/' . TRAVELPAYOUTS_REDUX_OPTION . '/enqueue', $this, 'overrideReduxCss');
        if (function_exists('wp_enqueue_editor')) {
            /** @see Admin::enqueue_block_editor_assets() */
            $loader->add_action('wp_enqueue_editor', $this, 'enqueue_block_editor_assets');
        } else {
            /** @see Admin::enqueue_block_editor_assets() */
            $loader->add_action('admin_enqueue_scripts', $this, 'enqueue_block_editor_assets');
        }
        /** @see Admin::landing_page_action() */

        $loader->add_action(
            'admin_action_' . LandingPage::ACTION,
            $this,
            'landing_page_action'
        );
        /** @see Admin::trackMarkerChanged() */

        $loader->add_action(
            'update_option_' . TRAVELPAYOUTS_REDUX_OPTION,
            $this,
            'trackMarkerChanged',
            10,
            2
        );
        /** @see Admin::pluginVersionAction() */
        $loader->add_action('admin_init', $this, 'pluginVersionAction');
        /** @see Admin::trackVersionChanged() */
        $loader->add_action(
            'update_option_' . TRAVELPAYOUTS_VERSION_KEY,
            $this,
            'trackVersionChanged',
            10,
            2
        );

        /** @see Admin::add_shortcodes_editor_btn_script() */
        $loader->add_filter('mce_external_plugins', $this, 'add_shortcodes_editor_btn_script');
        /** @see Admin::add_shortcodes_editor_btn() */
        $loader->add_filter('mce_buttons', $this, 'add_shortcodes_editor_btn');
        /** @see Admin::load_gutenberg_block() */
        $loader->add_action('init', $this, 'load_gutenberg_block');
    }

    public function load_redux_options()
    {
        if (Travelpayouts::getInstance()->currentUserCan('manage_options')) {
            OptionsInit::getInstance();
        }
    }

    /**
     * @param $old
     * @param $new
     */
    public function trackMarkerChanged($old, $new)
    {
        if (
            isset($old['account_api_marker'], $new['account_api_marker']) && $old['account_api_marker'] !== $new['account_api_marker']
        ) {
            Travelpayouts::snowTracker()->trackStructEvent(
                TrackerHelper::CATEGORY_INSTALL,
                TrackerHelper::ACTION_ACTIVATED,
                null,
                null,
                null,
                [
                    'marker' => $new['account_api_marker'],
                ]
            );
        }
    }

    /**
     * @param $old
     * @param $new
     */
    public function trackVersionChanged($old, $new)
    {
        if (version_compare($new, $old, '>')) {
            Travelpayouts::snowTracker()->trackStructEvent(
                TrackerHelper::CATEGORY_INSTALL,
                TrackerHelper::ACTION_UPDATED,
                null,
                null,
                null,
                [
                    'plugin_version_previous' => $old,
                ]
            );
        }
    }

    public function pluginVersionAction()
    {
        // Если есть версия прошлого плагина будет виден апдейт
        // TRAVELPAYOUTS_VERSION_KEY то же значение что и в прошлой версии
        $pluginOptionVersion = get_option(TRAVELPAYOUTS_VERSION_KEY);

        if (empty($pluginOptionVersion) || $pluginOptionVersion != TRAVELPAYOUTS_VERSION) {
            update_option(TRAVELPAYOUTS_VERSION_KEY, TRAVELPAYOUTS_VERSION);
        }
    }

    public function overrideReduxCss()
    {
        wp_dequeue_style('redux-admin-css');
    }

    public function enqueue_block_editor_assets()
    {
        try {
            Travelpayouts::getInstance()->assets->getAssetByName('admin-gutenberg-modal')->setInFooter(true)->enqueueStyle()->enqueueScript();
        } catch (RuntimeException $e) {
            Travelpayouts::logger()->error($e->getMessage());
        }
    }

    /**
     * Add <script> tag with JavaScript object
     * @since    1.0.0
     */
    public function localize_scripts()
    {
        wp_localize_script('jquery',
            $this->plugin_name . 'FrontApiSettings',
            [
                'nonce' => wp_create_nonce('wp_rest'),
                'endpoint' => esc_url(rest_url('/travelpayouts/front')),
                'locale' => LanguageHelper::dashboardLocale(),
                'settings_locale' => LanguageHelper::tableLocale(),
            ]
        );

    }

    public function migrate()
    {
        /**
         * import on plugin update
         * add_action( 'upgrader_process_complete', function( $upgrader_object, $options ) {
         *      import here
         * }, 10, 2 );
         */

        if (!Travelpayouts::getInstance()->currentUserCan('manage_options')) {
            die(Travelpayouts::__('Insufficient access rights!'));
        }

        $options = get_option(Migration::SOURCE_OPTION_NAME);
        $importDone = get_option(Migration::IMPORT_DONE_OPTION_NAME);

        if ($options && $importDone != Migration::IMPORT_DONE_TRUE) {
            $import = new Migration([
                'redux' => Travelpayouts::getRedux(),
                'source' => get_option(Migration::SOURCE_OPTION_NAME),
            ]);
            $import->import();
        }

        echo json_encode([
            'status' => 'success',
            'action' => 'reload',
        ]);
        die ();
    }

    public function migrateCancel()
    {
        if (!Travelpayouts::getInstance()->currentUserCan('manage_options')) {
            die(Travelpayouts::__('Insufficient access rights!'));
        }

        update_option(Migration::IMPORT_DONE_OPTION_NAME, Migration::IMPORT_DONE_TRUE);

        echo json_encode([
            'status' => 'success',
            'action' => 'reload',
        ]);
        die ();
    }

    public function importFile()
    {
        if (!Travelpayouts::getInstance()->currentUserCan('manage_options')) {
            die(Travelpayouts::__('Insufficient access rights!'));
        }

        if (isset($_POST['settings']) && !empty($_POST['settings'])) {
            $import = new Migration([
                'redux' => Travelpayouts::getRedux(),
                'source' => $_POST['settings'],
            ]);

            try {
                $import->import();
            } catch (Exception $exception) {
                Notices::addErrorNotice(Notices::REDUX_IMPORT_NOTICE, Travelpayouts::__('Import failed'));
                die ();
            }

        }

        Notices::addSuccessNotice(Notices::REDUX_IMPORT_NOTICE, Travelpayouts::__('Import completed'));
        die ();
    }

    public function get_landing_page()
    {
        return new Menu(
            new LandingPage($this),
            LandingModel::LANDING_SLUG,
            Travelpayouts::__('Travelpayouts WordPress plugin'),
            Travelpayouts::_x('Travelpayouts', 'Travelpayouts')
        );
    }

    public function admin_notices()
    {
        if (!Travelpayouts::getInstance()->currentUserCan('manage_options')) {
            return false;
        }

        // Добавляет welcome (плашку) уведомление
        $this->welcomeNotice();
        $this->importNotice();

        // Получает и отображает уведомления
        Notices::render();
    }

    public function landing_page_action()
    {
        if (!Travelpayouts::getInstance()->currentUserCan('manage_options')) {
            die(Travelpayouts::__('Insufficient access rights!'));
        }

        if (wp_verify_nonce($_POST['_wpnonce'], LandingPage::ACTION)) {
            $model = new LandingModel();
            $model->setSanitizedAttributes($_POST);
            $model->save();

            exit(wp_redirect($_POST['_wp_http_referer']));
        }

        die(Travelpayouts::__('WP nonce verification failed!'));
    }

    /**
     * Добавляет уведомление если redux plugin не активирован
     * пустое значения API token или affiliate marker в настройках аккаунта
     *
     * @return bool
     */
    private function welcomeNotice()
    {
        if (function_exists('get_current_screen')) {
            $page = get_current_screen();

            // не отображать на страницах landing, активация плагина
            $pages = [
                'plugins_page_install-required-plugins',
                'settings_page_' . LandingModel::LANDING_SLUG,
            ];
            if (in_array($page->base, $pages)) {
                return true;
            }
        }

        $account = Travelpayouts::getInstance()->account->section->data;
        if (empty($account->get('api_token')) || empty($account->get('api_marker'))) {
            return Notices::addStyledNotice(
                Notices::IS_ACTIVE_REDUX,
                Travelpayouts::__('Activate the Travelpayouts plugin'),
                Travelpayouts::__('Enter your Travelpayouts authorization details and start earning now'),
                wp_nonce_url(add_query_arg(['page' => LandingModel::LANDING_SLUG], admin_url('options-general.php'))),
                Travelpayouts::__('Activate the plugin')
            );
        }
    }

    private function importNotice()
    {
        $sourceOption = get_option(Migration::SOURCE_OPTION_NAME);
        $canImport = get_option(Migration::IMPORT_DONE_OPTION_NAME) != Migration::IMPORT_DONE_TRUE;

        if ($sourceOption && $canImport) {
            return Notices::addStyledImportNotice(
                Notices::REDUX_IMPORT_ACTION_NOTICE,
                Travelpayouts::__('Import settings from the previous version'),
                Travelpayouts::__('You updated the Travelpayotus WordPress plugin. This is a new version and has differences with your previous one. To be sure that your existing settings were imported correctly, you have to confirm and click the button on the right. Please double-check after importing.'),
                Travelpayouts::__('Import settings')
            );
        }
    }

    /**
     * @param $buttons
     * @return array
     */
    public function add_shortcodes_editor_btn($buttons)
    {
        $buttons = array_merge($buttons, [$this->plugin_name . '_shortcodes_btn']);

        return $buttons;
    }

    public function add_shortcodes_editor_btn_script($plugin_array)
    {
        $plugin_array[$this->plugin_name . '_shortcodes_editor_btn'] = Travelpayouts::getAlias('@webadmin/editor/travelpayouts-editor-shortcodes-modal.js');
        return $plugin_array;
    }

    public function load_gutenberg_block()
    {
        try {
            $gutenbergAsset = Travelpayouts::getInstance()
                ->assets
                ->getAssetByName('admin-gutenberg-injector')
                ->registerStyle(['wp-editor'], null)
                ->registerScript([
                    'wp-blocks',
                    'wp-i18n',
                    'wp-element',
                    'wp-editor',
                ],
                    true
                )->localizeScript(
                    $this->plugin_name . '_block_shortcodes_globals', // Array containing dynamic data for a JS Global.
                    [
                        'pluginDirPath' => plugin_dir_path(__DIR__),
                        'pluginDirUrl' => plugin_dir_url(__DIR__),
                    ]
                );
            /**
             * Register Gutenberg block on server-side.
             *
             * Register the block on server-side to ensure that the block
             * scripts and styles for both frontend and backend are
             * enqueued when the editor loads.
             *
             * @link https://wordpress.org/gutenberg/handbook/blocks/writing-your-first-block-type#enqueuing-block-scripts
             * @since 1.16.0
             */
            if (function_exists('register_block_type')) {
                register_block_type(
                    $this->plugin_name . '/shortcodes', [
                        'editor_style' => $gutenbergAsset->styleHandlerName,
                        'editor_script' => $gutenbergAsset->scriptHandlerName,
                    ]
                );
            }
        } catch (RuntimeException $e) {
            Travelpayouts::logger()->error($e->getMessage());
        }
    }
}
