<?php
/**
 * Created by: Andrey Polyakov (andrey@polyakov.im)
 */

namespace Travelpayouts\components\model;

use Exception;
use Travelpayouts\components\BaseInjectedObject;
use Travelpayouts\helpers\ArrayHelper;

/**
 * Class CollectionModel
 * @package Travelpayouts\components\model
 * @property-read array $data
 * @property-read int $totalItemsCount
 */
abstract class CollectionModel extends BaseInjectedObject
{
    protected $primaryKey = 'id';
    /**
     * @var mixed[]
     */
    private $_data;

    /**
     * @var int|null
     */
    private $_increment;

    /**
     * Возвращаем содержимое коллекции
     * @return array
     */
    abstract protected function getCollection();

    /**
     * Сохраняем изменения в виде json представления данных
     * @param string $value
     * @return bool
     */
    abstract protected function setCollection($value);

    /**
     * @return mixed[]
     * @see CollectionModel::getData()
     */
    public function all()
    {
        return $this->getData();
    }

    /**
     * Создаем запись
     * @param mixed $attributes
     * @return int|false
     * @throws Exception
     */
    public function create($attributes)
    {
        $newItem = $this->assignPrimaryKeyToItem($attributes);
        $this->setIncrement($this->getIncrement() + 1);

        if ($this->setData(array_merge($this->getData(), [
            $newItem,
        ]))) {
            return $newItem[$this->primaryKey];
        }
        return false;
    }

    /**
     * Обновляем запись
     * @param int $id
     * @param mixed $attributes
     * @return bool
     * @throws Exception
     */
    public function update($id, $attributes)
    {
        $id = (int)$id;
        $result = array_map(function ($item) use ($id, $attributes) {
            $itemId = isset($item[$this->primaryKey])
                ? $item[$this->primaryKey]
                : null;
            return $itemId !== $id
                ? $item
                : $this->assignPrimaryKeyToItem($attributes, $id);
        }, $this->getData());

        return $this->setData($result);
    }

    /**
     * Удаляем запись
     * @param $id
     * @return bool
     * @throws Exception
     * @see CollectionModel::deleteMany()
     */
    public function delete($id)
    {
        return $this->deleteMany([$id]);
    }

    /**
     * Удаляем записи по id
     * @param $idList
     * @return bool
     * @throws Exception
     */
    public function deleteMany($idList)
    {
        $data = $this->getData();

        $indexList = array_map(function ($id) {
            return $this->findKeyByValue('id', (int)$id);
        }, $idList);
        $indexList = array_filter($indexList, function ($index) {
            return is_int($index);
        });
        if (!empty($indexList)) {
            $filteredData = array_filter($data, static function ($itemPrimaryKey) use ($indexList) {
                return !in_array($itemPrimaryKey, $indexList, true);
            }, ARRAY_FILTER_USE_KEY);
            return $this->setData($filteredData);
        }
        return false;
    }

    /**
     * Просмотр записи
     * @param int $id
     * @return array|false
     */
    public function view($id)
    {
        return $this->findByPrimaryKey($id);
    }

    /**
     * Получаем количество элементов
     * @return int
     */
    public function getTotalItemsCount()
    {
        return count($this->getData());
    }

    /**
     * Получаем содержимое коллекции
     * @return array|mixed|mixed[]
     */
    public function getData()
    {
        if (!$this->_data) {
            $collection = $this->getCollection();
            if (is_array($collection) && isset($collection['data'])) {
                $this->_data = $collection['data'];
            } elseif (ArrayHelper::isIndexed($collection)) {
                $this->_data = $collection;
            } else {
                $this->_data = [];
            }
        }
        return $this->_data;
    }

    /**
     * @param int $value
     * @return array|false
     */
    public function findByPrimaryKey($value)
    {
        return $this->findByColumnValue('id', (int)$value);
    }

    /**
     * @param $column - колонка для поиска
     * @param $value - значение
     * @return false|array
     * @see self::findKeyByValue()
     */
    public function findByColumnValue($column, $value)
    {
        $key = $this->findKeyByValue($column, $value);
        $data = $this->getData();
        return $key !== false && isset($data[$key])
            ? $data[$key]
            : false;
    }


    /**
     * Добавляем primaryKey к аттрибутам и возвращаем
     * @param $attributes
     * @param null $primaryKey
     * @return array
     */
    protected function assignPrimaryKeyToItem($attributes, $primaryKey = null)
    {
        return array_merge($attributes, [
            $this->primaryKey => $primaryKey
                ?: $this->getIncrement(),
        ]);
    }

    protected function getIncrement()
    {
        if (!$this->_increment) {
            $collection = $this->getCollection();
            if (is_array($collection) && ArrayHelper::isAssociative($collection) && isset($collection['increment'])) {
                $this->_increment = (int)$collection['increment'];
            } else {
                $this->_increment = 1;
            }
        }
        return $this->_increment;
    }

    private function setIncrement($value)
    {
        $this->_increment = $value;
    }

    private function setData($value)
    {
        if (is_array($value) && ArrayHelper::isIndexed($value)) {
            $data = json_encode([
                'increment' => $this->getIncrement(),
                'data' => array_values($value),
            ]);
            $this->_data = null;
            $result = $this->setCollection($data);
            if (!is_bool($result)) {
                $calledClass = get_called_class();
                throw new Exception("$calledClass::setCollection method must return boolean");
            }
            return $result;
        }
        throw new Exception('Value must be an indexed array');
    }

    /**
     * @param $column - колонка для поиска
     * @param $value - значение
     * @return false|int
     */
    protected function findKeyByValue($column, $value)
    {
        return array_search($value, array_column($this->getData(), $column), true);
    }
}
