<?php
/**
 * Created by: Andrey Polyakov (andrey@polyakov.im)
 */

namespace Travelpayouts\helpers;


class ArrayHelper
{
    /***
     * Полифил для метода array_key_first
     * @param $array
     * @return int|string|null
     */
    public static function getFirstKey($array)
    {
        foreach ($array as $key => $unused) {
            return $key;
        }
        return null;
    }

    /**
     * Returns a value indicating whether the given array is an associative array.
     * An array is associative if all its keys are strings. If `$allStrings` is false,
     * then an array will be treated as associative if at least one of its keys is a string.
     * Note that an empty array will NOT be considered associative.
     * @param array $array the array being checked
     * @param bool $allStrings whether the array keys must be all strings in order for
     * the array to be treated as associative.
     * @return bool whether the array is associative
     */
    public static function isAssociative($array, $allStrings = true)
    {
        if (!is_array($array) || empty($array)) {
            return false;
        }

        if ($allStrings) {
            foreach ($array as $key => $value) {
                if (!is_string($key)) {
                    return false;
                }
            }

            return true;
        }

        foreach ($array as $key => $value) {
            if (is_string($key)) {
                return true;
            }
        }

        return false;
    }

    /**
     * Returns a value indicating whether the given array is an indexed array.
     * An array is indexed if all its keys are integers. If `$consecutive` is true,
     * then the array keys must be a consecutive sequence starting from 0.
     * Note that an empty array will be considered indexed.
     * @param array $array the array being checked
     * @param bool $consecutive whether the array keys must be a consecutive sequence
     * in order for the array to be treated as indexed.
     * @return bool whether the array is indexed
     */
    public static function isIndexed($array, $consecutive = false)
    {
        if (!is_array($array)) {
            return false;
        }

        if (empty($array)) {
            return true;
        }

        if ($consecutive) {
            return array_keys($array) === range(0, count($array) - 1);
        }

        foreach ($array as $key => $value) {
            if (!is_int($key)) {
                return false;
            }
        }

        return true;
    }

    /**
     * @param $array
     * @param string $delimiter
     * @param string $prefix
     * @return array
     */
    public static function flatten($array, $delimiter = '.', $prefix = '')
    {
        $result = [];

        foreach ($array as $key => $value) {
            $new_key = $prefix . (empty($prefix)
                    ? ''
                    : $delimiter) . $key;
            if (is_array($value) && self::isAssociative($value)) {
                $result = array_merge($result, self::flatten($value, $delimiter, $new_key));
            } else {
                $result[$new_key] = $value;
            }
        }
        return $result;
    }

    /**
     * Находим первое значение из array_filter
     * @param array $input
     * @param null $callback
     * @param int $flag
     * @return mixed|null
     * @see array_filter()
     */
    public static function find(array $input, $callback = null, $flag = 0)
    {
        if (is_array($input) && is_callable($callback)) {
            $foundValues = array_filter($input, $callback, $flag);
            return count($foundValues)
                ? array_shift($foundValues)
                : null;
        }
        return null;
    }

    /**
     * @param mixed[]
     * @return mixed|null
     */
    public static function getFirst($array)
    {
        if (is_array($array) && count($array)) {
            $newArray = array_merge([], $array);
            return array_shift($newArray);
        }
        return null;
    }


    /**
     * Получаем значение из массива и приводим его к boolean
     * @param $array
     * @param $key
     * @return bool
     */
    public static function getBooleanValue($array, $key)
    {
        return $array && isset($array[$key])
            ? filter_var($array[$key], FILTER_VALIDATE_BOOLEAN)
            : false;
    }

    /**
     * Фильтруем пустые значения
     * @param $array
     * @return array
     */
    public static function filterEmpty($array)
    {
        return array_filter($array, function ($value) {
            return !is_null($value) && $value !== '';
        });
    }
}
