<?php

/**
 * The file that defines the core plugin class
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 * @link       http://www.travelpayouts.com/?locale=en
 * @since      1.0.0
 * @package    Travelpayouts
 * @subpackage Travelpayouts/includes
 */

use Travelpayouts\components\Assets;
use Travelpayouts\components\LanguageHelper;
use League\Plates\Engine;
use Rollbar\RollbarLogger;
use Travelpayouts\admin\Admin;
use Travelpayouts\components\base\cache\Cache;
use Travelpayouts\components\Component;
use Travelpayouts\components\module\ModuleRedux;
use Travelpayouts\components\multilingual\MultiLang;
use Travelpayouts\components\rest\FrontRestRoutes;
use Travelpayouts\components\traits\DITrait;
use Travelpayouts\components\traits\SingletonTrait;
use Travelpayouts\components\Translator;
use Travelpayouts\frontend\Frontend;
use Travelpayouts\includes\I18n;
use Travelpayouts\includes\HooksLoader;
use Travelpayouts\includes\ReduxConfigurator;
use Travelpayouts\includes\Router;
use Travelpayouts\includes\traits\AliasesTrait;
use Travelpayouts\includes\traits\I10nTrait;
use Travelpayouts\modules\account\Account;
use Travelpayouts\modules\linkForms;
use Travelpayouts\modules\links;
use Travelpayouts\modules\localization\Localization;
use Travelpayouts\modules\searchForms\Search;
use Travelpayouts\modules\settings\Settings;
use Travelpayouts\modules\tables\Tables;
use Travelpayouts\modules\widgets\Widgets;
use Travelpayouts\components\snowplow\Tracker;

/**
 * The core plugin class.
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 * @since      1.0.0
 * @package    Travelpayouts
 * @subpackage Travelpayouts/includes
 * @author     travelpayouts < wpplugin@travelpayouts.com>
 * @property-read Tables $tables
 * @property-read Widgets $widgets
 * @property-read Search $searchForms
 * @property-read linkForms\Links $linksForms
 * @property-read links\Links $links
 * @property-read Account $account
 * @property-read Settings $settings
 * @property-read Localization $localization
 * @property-read Cache $cache;
 * @property-read Translator $translator;
 * @property-read Frontend $frontend
 * @property-read Admin $admin
 * @property-read FrontRestRoutes $rest
 * @property-read RollbarLogger $rollbar
 * @property-read ReduxConfigurator $redux
 * @property-read Engine $template
 * @property-read Tracker $snowTracker
 * @property-read MultiLang $multiLang
 * @property-read Router $router
 * @property-read Assets $assets
 */
class Travelpayouts extends Component
{
    use DITrait;
    use SingletonTrait;
    use I10nTrait;
    use AliasesTrait;

    /**
     * The loader that's responsible for maintaining and registering all hooks that power
     * the plugin.
     * @since    1.0.0
     * @access   protected
     * @var      HooksLoader Maintains and registers all hooks for the plugin.
     */
    public static $loader;

    /**
     * The unique identifier of this plugin.
     * @since    1.0.0
     * @access   protected
     * @var      string $plugin_name The string used to uniquely identify this plugin.
     */
    protected $plugin_name;

    /**
     * The current version of the plugin.
     * @since    1.0.0
     * @access   protected
     * @var      string $version The current version of the plugin.
     */
    protected $version;

    /**
     * @var array registered path aliases
     * @see  AliasesTrait::getAlias()
     * @see  AliasesTrait::setAlias()
     */
    public static $aliases = [
        '@root' => TRAVELPAYOUTS_PLUGIN_PATH,
        '@data' => TRAVELPAYOUTS_PLUGIN_PATH . 'data',
        '@src' => TRAVELPAYOUTS_PLUGIN_PATH . 'src',
        '@config' => TRAVELPAYOUTS_PLUGIN_PATH . 'src/config',
        '@includes' => TRAVELPAYOUTS_PLUGIN_PATH . 'src/includes',
        '@admin' => TRAVELPAYOUTS_PLUGIN_PATH . 'src/admin',
        '@frontend' => TRAVELPAYOUTS_PLUGIN_PATH . 'src/frontend',
        '@assets' => TRAVELPAYOUTS_PLUGIN_PATH . 'assets',
        '@images' => TRAVELPAYOUTS_PLUGIN_PATH . 'images',
    ];

    public function __construct()
    {
        $this->registerAliases();
        $this->version = TRAVELPAYOUTS_VERSION;
        $this->plugin_name = 'travelpayouts';
        self::$_instances[__CLASS__] = $this;
        self::$loader = new HooksLoader();

        $containerDefinitions = $this->getContainerDefinitions();
        $this->createContainer($containerDefinitions);
        $this->loadModules();
        $this->setUpLocale();

        $this->translator->locale = LanguageHelper::tableLocale();

        $this->assets->loader->setUpHooks();
        $this->admin->setUpHooks();
        $this->frontend->setUpHooks();
        $this->registerShortcodes();
        $this->rest->setUpHooks();
    }

    /**
     * Регистрируем алиасы для приложения
     * @throws Exception
     */
    protected function registerAliases()
    {
        $uploadDir = wp_upload_dir();
        $aliasList = [
            [
                'name' => '@webhome',
                'path' => get_home_url(),
            ],
            [
                'name' => '@webroot',
                'path' => plugin_dir_url(self::getAlias('@src')),
            ],
            [
                'name' => '@webadmin',
                'path' => '@webroot/src/admin',
            ],
            [
                'name' => '@uploads',
                'path' => !$uploadDir['error']
                    ? $uploadDir['basedir'] . DIRECTORY_SEPARATOR . TRAVELPAYOUTS_PLUGIN_NAME
                    : null,
            ],
            [
                'name' => '@webuploads',
                'path' => !$uploadDir['error']
                    ?
                    $uploadDir['baseurl'] . DIRECTORY_SEPARATOR . TRAVELPAYOUTS_PLUGIN_NAME
                    : null,
            ],
            [
                'name' => '@runtime',
                'path' => '@uploads',
            ],
            [
                'name' => '@webImages',
                'path' => '@webroot/images',
            ],
        ];

        foreach ($aliasList as $alias) {
            if (isset($alias['name'], $alias['path'])) {
                self::setAlias($alias['name'], $alias['path']);
            }
        }
    }

    /**
     * @return array
     * @throws Exception
     */
    private function getContainerDefinitions()
    {
        $configPath = self::getAlias('@config/main.php');
        if (file_exists($configPath)) {
            return require($configPath);
        }
        throw new Exception(self::__('Config path: {configPath} does not exist', ['configPath' => $configPath]));
    }

    /**
     * Загружаем все модули, указанные в /config/config.php
     */
    private function loadModules()
    {
        if (self::container()->has('config')) {
            $config = self::container()->get('config');
            if (isset($config['modules'])) {
                $this->add_components($config['modules']);
            }
        }
    }

    /**
     * Define the locale for this plugin for internationalization.
     * Uses the I18n class in order to set the domain and to register the hook
     * with WordPress.
     * @since    1.0.0
     */
    private function setUpLocale()
    {
        $i18n = new I18n([
            'domain' => TRAVELPAYOUTS_TEXT_DOMAIN,
            'localePath' => self::getAlias('@root/languages'),
        ]);
        $i18n->register();
    }

    public function currentUserCan($capability)
    {
        return current_user_can($capability);
    }

    private function registerShortcodes()
    {
        /**
         * @var ModuleRedux[] $modulesWithShortcodes
         */
        $modulesWithShortcodes = [
            $this->tables,
            $this->widgets,
            $this->links,
            $this->searchForms,
        ];

        foreach ($modulesWithShortcodes as $module) {
            $module->registerShortcodes();
        }
    }

    /**
     * Run the loader to execute all of the hooks with WordPress.
     * @since    1.0.0
     */
    public function run()
    {
        self::loader()->run();
    }

    /**
     * @return ReduxConfigurator
     */
    public static function getRedux()
    {
        return self::getInstance()->redux;
    }

    /**
     * The reference to the class that orchestrates the hooks with the plugin.
     * @return    HooksLoader    Orchestrates the hooks of the plugin.
     * @since     1.0.0
     */
    public static function loader()
    {
        return self::$loader;
    }

    public static function container()
    {
        return self::getInstance()->get_container();
    }

    public static function isReduxActivated()
    {
        return class_exists('Redux');
    }

    public static function t($id, array $parameters = [], $domain = null, $locale = null)
    {
        return self::getInstance()->translator->trans($id, $parameters, $domain, $locale);
    }

    public static function tChoise($id, $number, array $parameters = [], $domain = null, $locale = null)
    {
        return self::getInstance()->translator->transChoice($id, $number, $parameters, $domain, $locale);
    }

    /**
     * Получаем локаль из translator компонента
     * @param bool $fallbackLocale - возвращает оригинальный код локали или код фоллбек языка
     * @return mixed|string
     */
    public static function locale($fallbackLocale = true)
    {
        return Travelpayouts::getInstance()->translator
            ? Travelpayouts::getInstance()->translator->getLocale($fallbackLocale)
            : Translator::ENGLISH;
    }

    /**
     * @return Cache
     */
    public function getCache()
    {
        return self::getInstance()->cache;
    }

    /**
     * @return RollbarLogger
     */
    public static function logger()
    {
        return self::getInstance()->rollbar;
    }

    /**
     * @return Engine
     */
    public static function template()
    {
        return self::getInstance()->template;
    }

    /**
     * @return Tracker
     */
    public static function snowTracker()
    {
        return self::getInstance()->snowTracker;
    }

    public static function multiLang()
    {
        return self::getInstance()->multiLang;
    }
}
