<?php
/**
 * Created by: Andrey Polyakov (andrey@polyakov.im)
 */

namespace Travelpayouts\modules\searchForms\components;

use Adbar\Dot;
use DI\Annotation\Inject;
use Travelpayouts;
use Travelpayouts\components\HtmlHelper as Html;
use Travelpayouts\components\shortcodes\ShortcodeModel;
use Travelpayouts\modules\account\Account;
use Travelpayouts\modules\searchForms\components\models\SearchFormsModel;

/**
 * Class SearchShortcodes
 * @package Travelpayouts\src\modules\searchForms\components
 * @property-read string $widget_id
 * @property-read Dot|null $widget_params
 * @property-read string $widget_url
 * @property-read string $widget_type;
 * @property-read Dot $item_data;
 * @property-read string $origin_iata;
 * @property-read string $destination_iata;
 * @property-read array $hotel_city_data;
 */
class SearchShortcodes extends ShortcodeModel
{

    const TYPE_AVIA = 'avia';
    const TYPE_HOTEL = 'hotel';
    const TYPE_AVIA_HOTEL = 'avia_hotel';
    const FORM_CODE_REGEXP = '/window.TP_FORM_SETTINGS\["(?<widgetId>[\w]+)"\](\s?=\s?)(?<widgetParams>[^;]+)/';

    public $code_form;

    public $id;
    public $slug;
    public $type = 'avia';
    public $origin;
    public $destination;
    public $hotel_city;
    public $subid;

    protected $_item_data;
    protected $_widgetParams;
    protected $_widgetId;
    protected $_widgetUrl;
    protected $_widget_type;
    protected $_marker;

    /**
     * @Inject("account")
     * @var Account
     */
    protected $accountModule;

    public function rules()
    {
        return [
            [
                ['id'],
                'id_or_slug_validator',
                'params' => [
                    'attributeList' => [
                        'id',
                        'slug',
                    ],
                ],
                'skipOnEmpty' => false,
            ],
            [
                [
                    'origin',
                    'destination',
                    'hotel_city',
                    'subid',
                    'type',
                ],
                'string',
            ],
            [
                [
                    'origin',
                    'destination',
                ],
                'string',
                'min' => 3,
                'max' => 3,
            ],
            [
                ['hotel_city'],
                'hotel_city_validator',
            ],
        ];
    }

    public function id_or_slug_validator($attribute, $params)
    {
        if (isset($params['attributeList'])) {
            $hasValidAttributes = array_map(function ($key) {
                return (bool)$this->$key;
            }, $params['attributeList']);

            $isValid = in_array(true, $hasValidAttributes, true);

            if (!$isValid) {
                $attributesAsString = implode(', ', $params['attributeList']);
                $this->add_error($attribute, Travelpayouts::__('One of parameters ({attributes}) is required', [
                    'attributes' => $attributesAsString,
                ]));
            }
        }
    }

    public function hotel_city_validator($attribute)
    {
        if ($this->$attribute && !preg_match('/(?<hotels_count>\d+)?,\s?(?<search_id>\d+),\s?(?<search_type>hotel|city),\s?(?<country_name>.+)/', $this->$attribute)) {
            $this->add_error($attribute, Travelpayouts::__('{attribute} has invalid value', [
                'attribute' => $this->get_attribute_label($attribute),
            ]));
        }
    }

    protected function get_item_data()
    {
        if (!$this->_item_data) {
            $model = new SearchFormsModel();
            $data = null;
            if ($this->id) {
                $data = $model->view($this->id);
            } elseif ($this->slug) {
                $data = $model->findByColumnValue('slug', $this->slug);
            }

            $this->_item_data = new Dot($data);
        }

        return $this->_item_data;
    }


    public function render()
    {
        if ($this->validate()) {
            $data = $this->item_data;
            if (!$data->isEmpty()) {
                $isValidCodeForm = $this->is_valid_code_form($data['code_form']);
                $this->code_form = $data['code_form'];
                if ($isValidCodeForm) {
                    if ($this->widget_params && $this->enrich_widget_params()) {
                        return $this->render_script();
                    }
                } else {
                    return $this->render_raw_content();
                }
            }
        }
        return '';
    }

    protected function enrich_widget_params()
    {
        if ($this->widget_params->isEmpty()) return false;

        switch ($this->widget_type) {
            case self::TYPE_AVIA:
                $this->widget_params->delete(['hotel']);
                break;
            case self::TYPE_HOTEL:
                $this->widget_params->delete([
                    'origin',
                    'destination',
                ]);
                break;
        }

        if ($this->widget_params->has('origin')) {
            $this->widget_params->set('origin', [
                'iata' => $this->origin_iata,
            ]);
        }

        if ($this->widget_params->has('destination')) {
            $this->widget_params->set('destination', [
                'iata' => $this->destination_iata,
            ]);
        }

        if ($this->widget_params->has('hotel')) {
            $hotelData = $this->hotel_city_data;
            if (is_string($hotelData)) {
                $hotelData = json_decode($hotelData);
                // TODO убрать повторное преобразование
                // $this->hotel_city_data был преобразован в json дважды
                if (is_string($hotelData)) {
                    $hotelData = json_decode($hotelData, true);
                }
            }
            $this->widget_params->set('hotel', $hotelData);
        }

        $marker = $this->get_marker();
        if ($marker) {
            if ($this->widget_params->has('marker')) {
                $this->widget_params->set('marker', $marker);
            }

            if ($this->widget_params->has('best_offer.marker')) {
                $this->widget_params->set('best_offer.marker', $marker);
            }
        }
        return true;
    }

    /**
     * Парсим code_form и получаем widget_id
     * @return mixed
     */
    public function get_widget_id()
    {
        if (!$this->_widgetId && $this->code_form && preg_match(self::FORM_CODE_REGEXP, $this->code_form, $widgetParamMatches)) {
            $this->_widgetId = $widgetParamMatches['widgetId'];
        }
        return $this->_widgetId;
    }

    /**
     * Парсим code_form и получаем параметры виджета
     * @return Dot
     */
    public function get_widget_params()
    {
        if (!$this->_widgetParams && $this->code_form && preg_match(self::FORM_CODE_REGEXP, $this->code_form, $widgetParamMatches)) {
            $decodedParams = json_decode($widgetParamMatches['widgetParams'], true);

            $this->_widgetParams = new Dot(
                $decodedParams
                    ?: []
            );
        }
        return $this->_widgetParams;
    }

    public function is_valid_code_form($data)
    {
        return $data && is_string($data) && preg_match(self::FORM_CODE_REGEXP, $data);
    }

    /**
     * Парсим code_form и получаем url js файла
     * @return mixed
     */
    public function get_widget_url()
    {
        if (!$this->_widgetUrl && $this->code_form && preg_match('/src="(?<widgetUrl>.+)"/', $this->code_form, $widgetUrlMatches)) {
            $this->_widgetUrl = $widgetUrlMatches['widgetUrl'];
        }
        return $this->_widgetUrl;
    }

    public function get_widget_type()
    {
        if (!$this->_widget_type && $this->code_form) {
            $this->_widget_type = $this->widget_params->get('form_type');
        }
        return $this->_widget_type;
    }

    public function get_marker()
    {
        if (!$this->_marker) {


            $subIdParams = array_filter([
                $this->accountModule->marker,
                $this->subid,
            ]);

            $this->_marker = implode('.', $subIdParams);
        }
        return $this->_marker;
    }

    protected function render_script()
    {
        if (!$this->widget_params->isEmpty() && $this->widget_url && $this->widget_id) {
            $widgetId = $this->widget_id;
            $widgetParams = json_encode($this->widget_params->all());

            $scriptParams = <<<JS
window.TP_FORM_SETTINGS = window.TP_FORM_SETTINGS || {};
window.TP_FORM_SETTINGS["$widgetId"] = $widgetParams;
JS;

            return implode('', [
                Html::script($scriptParams),
                Html::scriptFile($this->widget_url, ['async' => 'async']),
            ]);
        }
        return '';
    }

    protected function render_raw_content()
    {
        return $this->code_form;
    }

    public function render_shortcode($attributes, $content = null, $tag = '')
    {
        $model = new self();
        $model->attributes = $attributes;
        return $model->render();
    }

    public function get_origin_iata()
    {
        return $this->origin
            ?: $this->item_data->get('from_city.code');
    }

    public function get_destination_iata()
    {
        return $this->destination
            ?: $this->item_data->get('to_city.code');
    }

    public function get_hotel_city_data()
    {
        if ($this->hotel_city && is_string($this->hotel_city)) {
            $value = $this->parse_hotel_city_string($this->hotel_city);
            if ($value) {
                return $value;
            }
        }
        return $this->item_data->get('hotel_city');
    }

    /**
     * Парсим строку из аттрибутов шорткода, так как в первой версии плагина используется разделитель "запятая"
     * (а в названии переменных она может присутствовать)
     * делаем это весьма необычным способом (вместо обычного explode())
     * @param $value
     * @return array|null
     */
    protected function parse_hotel_city_string($value)
    {
        if (preg_match('/(?<hotels_count>\d+)?,\s?(?<search_id>\d+),\s?(?<search_type>hotel|city),\s?(?<country_name>.+)/', $value, $hotelCityValues)) {
            $values = array_filter($hotelCityValues, static function ($key) {
                return !is_int($key);
            }, ARRAY_FILTER_USE_KEY);

            $placeNameString = preg_replace('/' . $hotelCityValues[0] . '/', '', $value);
            $placeNameValues = explode(',', $placeNameString);
            $placeNameValuesMapped = array_filter(
                array_map(static function ($value) {
                    return strlen($value) > 0
                        ? trim($value)
                        : null;
                }, $placeNameValues)
            );

            $placeNameValues = [
                'name' => '',
                'location' => '',
            ];

            if (count($placeNameValuesMapped) >= 2) {
                $placeNameValues = [
                    'name' => $placeNameValuesMapped[0],
                    'location' => $placeNameValuesMapped[1],
                ];
            }

            return array_merge($values, $placeNameValues);
        }
        return null;
    }

    /**
     * @inheritDoc
     */
    public static function shortcodeTags()
    {
        return ['tp_search_shortcodes'];
    }

    /**
     * @inheritDoc
     */
    public static function render_shortcode_static($attributes = [], $content = null, $tag = '')
    {
        $model = new self();
        $model->attributes = $attributes;
        return $model->render();
    }
}
